require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_runprofile',
  dscmeta_resource_friendly_name: 'RunProfile',
  dscmeta_resource_name: 'MimSyncRunProfile',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'MimDsc',
  dscmeta_module_version: '0.0.1',
  docs: 'The DSC RunProfile resource type.
         Automatically generated from version 0.0.1',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_creationtime: {
      type: 'Optional[String]',
      desc: ' ',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_id: {
      type: 'Optional[String]',
      desc: ' ',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_managementagentname: {
      type: 'String',
      desc: ' ',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: ' ',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_runsteps: {
      type: 'Optional[Array[Struct[{
              inputfile => Optional[String],
              timeout => Optional[Integer[0, 4294967295]],
              batchsize => Optional[Integer[0, 4294967295]],
              pagesize => Optional[Integer[0, 4294967295]],
              steptype => Optional[String],
              fakeidentifier => Optional[String],
              dropfilename => Optional[String],
              stepsubtype => Optional[Array[String]],
              partitionidentifier => Optional[String],
              objectdeletelimit => Optional[Integer[0, 4294967295]],
              objectprocesslimit => Optional[Integer[0, 4294967295]],
              logfilepath => Optional[String]
            }]]]',
      desc: 'Array of Run Steps',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MimSyncRunStep[]',
      mof_is_embedded: true,
    },
    dsc_lastmodificationtime: {
      type: 'Optional[String]',
      desc: ' ',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_version: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: ' ',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
  },
)
